package eu.maiora.db

import com.fasterxml.jackson.databind.deser.impl.CreatorCandidate.Param
import eu.maiora.model.Accounts
import eu.maiora.model.Parametri
import kotlinx.coroutines.Dispatchers
import org.jetbrains.exposed.dao.IntEntity
import org.jetbrains.exposed.dao.IntEntityClass
import org.jetbrains.exposed.dao.id.EntityID
import org.jetbrains.exposed.dao.id.IdTable
import org.jetbrains.exposed.sql.Transaction
import org.jetbrains.exposed.sql.transactions.experimental.newSuspendedTransaction

object AccountsTable : IdTable<Int>("accounts"){
    override val id = integer("id").entityId()
    val username = varchar("username", 255)
    val password = varchar("password", 255)

    override val primaryKey = PrimaryKey(id)
}

object ParametriTable : IdTable<Int>("parametri"){
    override val id = integer("id").entityId()
    val chiave = varchar("chiave", 255)
    val valore = varchar("valore", 255)

    override val primaryKey = PrimaryKey(id)
}

class AccountsDAO(id: EntityID<Int>) :IntEntity(id) {
    companion object : IntEntityClass<AccountsDAO>(AccountsTable)

    var username by AccountsTable.username
    var password by AccountsTable.password
}

class ParametriDAO(id: EntityID<Int>) :IntEntity(id) {
    companion object : IntEntityClass<ParametriDAO>(ParametriTable)

    var chiave by ParametriTable.chiave
    var valore by ParametriTable.valore
}


fun accountsDaoToModel(dao: AccountsDAO) = Accounts(
    dao.id.value,
    dao.username,
    dao.password
)

fun parametriDaoToModel(dao: ParametriDAO) = Parametri(
    dao.id.value,
    dao.chiave,
    dao.valore
)


suspend fun <T> suspendTransaction(block: Transaction.() -> T): T =
    newSuspendedTransaction(Dispatchers.IO, statement = block)