package eu.maiora.plugins

import com.auth0.jwt.JWT
import com.auth0.jwt.algorithms.Algorithm
import io.ktor.http.*
import io.ktor.server.application.*
import io.ktor.server.auth.*
import io.ktor.server.auth.jwt.*
import io.ktor.server.response.*

fun Application.configureSecurity(secret: String) {
    install(Authentication) {
        jwt ("auth-jwt"){
            verifier(
                JWT
                    .require(Algorithm.HMAC256(secret))
                    .build())
            validate { credential ->
                val expiresAt = credential.payload.expiresAt?.time ?: 0
                val now = System.currentTimeMillis()

                // Verifica se il token ? scaduto
                if (expiresAt >= now) {
                    JWTPrincipal(credential.payload)
                }
                else {
                    null
                }
            }
            challenge { defaultScheme, realm ->
                call.respond(HttpStatusCode.Unauthorized, "Token non valido o scaduto")
            }
        }
    }
}