package eu.maiora.routes

import com.auth0.jwt.JWT
import com.auth0.jwt.algorithms.Algorithm
import eu.maiora.model.AccountsRepositoryImpl
import io.ktor.http.*
import io.ktor.server.application.*
import io.ktor.server.config.*
import io.ktor.server.request.*
import io.ktor.server.response.*
import io.ktor.server.routing.*
import kotlinx.serialization.Serializable
import kotlinx.serialization.json.Json
import org.slf4j.LoggerFactory
import java.util.*


fun Route.auth(accountsRepository: AccountsRepositoryImpl) {
    route("/auth") {
        post() {
            // Riceve il body della richiesta e lo deserializza in ReceivedResponse
            val receivedResponse = try {
                call.receive<ReceivedResponse>()
            } catch (e: Exception) {
                return@post call.respondText(
                    "Body mancante " + e.stackTraceToString(),
                    status = HttpStatusCode.BadRequest
                )
            }
            val logger = LoggerFactory.getLogger("AuthRoute")
            logger.info(
                "param: " +
                        receivedResponse.param
            );

            // Decodifica la stringa da Base64 a oggetto Credentials
            val decodedBytes = Base64.getDecoder().decode(receivedResponse.param)
            val decodedString = String(decodedBytes)
            val credentials = try {
                Json.decodeFromString<Credentials>(decodedString)
            } catch (e: Exception) {
                return@post call.respondText(
                    "Errore nel param. Verificare la codifica. \n" + e.stackTraceToString(),
                    status = HttpStatusCode.BadRequest
                )
            }

            //verifica credenziali (recupero account dal database)
            val account = accountsRepository.accountByUsername(credentials.username)
            //se le credenziali sono valide, creare il JWT
            if (account != null) {
                val passwordPlain = String(
                    Base64.getDecoder().decode(
                        StringBuffer(
                            String(
                                Base64.getDecoder().decode(account.password.toByteArray())
                            )
                        ).reverse().toString().toByteArray()
                    )
                )
                if (passwordPlain.equals(credentials.password)) {
                    val config = ApplicationConfig("application.conf")
                    val secret = config.property("ktor.jwt.secret").getString()

                    val now = System.currentTimeMillis()
                    val expiration = now + 86400000 // Scadenza tra 1 giorno (24 ore)

                    val token = JWT.create()
                        .withSubject(account.username)
                        .withIssuedAt(Date(now))
                        .withExpiresAt(Date(expiration))
                        .sign(Algorithm.HMAC256(secret))

                    // Risponde con la stringa decodificata
                    call.respond(HttpStatusCode.OK, token)


                } else {
                    call.respond(HttpStatusCode.Unauthorized)
                }
            } else {
                call.respond(HttpStatusCode.Unauthorized)
            }

        }
    }


}

@Serializable
data class ReceivedResponse(val param: String)

@Serializable
data class Credentials(val username: String, val password: String)